@@ -26,7 +26,6 @@
 #include <QIcon>
 #include <QModelIndex>
 #include <QCoreApplication>
-#include <QDBusInterface>
 #include <QSqlQuery>
 #include <QSqlError>
 #include <QUuid>
@@ -53,6 +52,7 @@
 #include "utils/range.h"
 #include "utils/continue_with.h"
 #include "utils/dbusfuture_p.h"
+#include "common/dbus/common.h"
 
 #define ACTIVITY_COLUMN 0
 #define AGENT_COLUMN    1
@@ -68,9 +68,7 @@ namespace Imports {
 class ResourceModel::LinkerService: public QDBusInterface {
 private:
     LinkerService()
-        : QDBusInterface("org.kde.ActivityManager",
-                         "/ActivityManager/Resources/Linking",
-                         "org.kde.ActivityManager.ResourcesLinking")
+        : KAMD_DBUS_INTERFACE(Resources/Linking, ResourcesLinking, Q_NULLPTR)
     {
     }
 
@@ -187,7 +185,7 @@ QHash<int, QByteArray> ResourceModel::roleNames() const
     return {
         { Qt::DisplayRole,    "display" },
         { Qt::DecorationRole, "decoration" },
-        { ResourceRole,       "url" },
+        { ResourceRole,       "uri" },
         { AgentRole,          "agent" },
         { ActivityRole,       "activity" },
         { DescriptionRole,    "subtitle" }
@@ -333,16 +331,16 @@ QVariant ResourceModel::data(const QModelIndex &proxyIndex, int role) const
 
     if (role == Qt::DisplayRole || role == DescriptionRole
             || role == Qt::DecorationRole) {
-        auto url = dataForColumn(index, RESOURCE_COLUMN).toString();
+        auto uri = dataForColumn(index, RESOURCE_COLUMN).toString();
 
         // TODO: Will probably need some more special handling -
         //       for application:/ and a few more
 
-        if (url.startsWith('/')) {
-            url = QStringLiteral("file://") + url;
+        if (uri.startsWith('/')) {
+            uri = QStringLiteral("file://") + uri;
         }
 
-        KFileItem file(url);
+        KFileItem file(uri);
 
         if (file.mimetype() == "application/x-desktop") {
             KDesktopFile desktop(file.localPath());
