@@ -1,5 +1,5 @@
 /*
- *   Copyright (C) 2012 Ivan Cukic <ivan.cukic(at)kde.org>
+ *   Copyright (C) 2012, 2013, 2014 Ivan Cukic <ivan.cukic(at)kde.org>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License version 2,
@@ -21,13 +21,12 @@
 
 #include <QDebug>
 #include <QMenu>
-#include <QDeclarativeContext>
-#include <QDeclarativeEngine>
-#include <QDeclarativeComponent>
-#include <QDBusInterface>
+#include <QQmlContext>
+#include <QQmlEngine>
+#include <QQmlComponent>
 #include <QDBusPendingCall>
 
-#include <QGraphicsObject>
+#include <QQuickView>
 
 #include <KLocalizedString>
 #include <KAboutData>
@@ -35,17 +34,16 @@
 #include <KPluginInfo>
 #include <KService>
 #include <KServiceTypeTrader>
-#include <kdeclarative.h>
-
-#include <Plasma/PackageStructure>
 
 #include "ui_MainConfigurationWidgetBase.h"
 #include "BlacklistedApplicationsModel.h"
 
 #include <utils/d_ptr_implementation.h>
 
+#include "kactivities-features.h"
+#include "common/dbus/common.h"
+
 K_PLUGIN_FACTORY(ActivitiesKCMFactory, registerPlugin<MainConfigurationWidget>();)
-K_EXPORT_PLUGIN(ActivitiesKCMFactory("kcm_activities", "kcm_activities"))
 
 class MainConfigurationWidget::Private : public Ui::MainConfigurationWidgetBase {
 public:
@@ -55,24 +53,17 @@ public:
     KSharedConfig::Ptr pluginConfig;
     KPluginSelector *pluginSelector;
     BlacklistedApplicationsModel *blacklistedApplicationsModel;
-    KDeclarative kdeclarative;
-    Plasma::PackageStructure::Ptr structure;
-    QGraphicsObject *viewBlacklistedApplicationsRoot;
+
+    QObject *viewBlacklistedApplicationsRoot;
+    QQuickView *viewBlacklistedApplications;
 };
 
 MainConfigurationWidget::MainConfigurationWidget(QWidget *parent, QVariantList args)
-    : KCModule(ActivitiesKCMFactory::componentData(), parent)
+    : KCModule(parent, args)
     , d()
 {
     Q_UNUSED(args)
 
-    const auto about = new KAboutData(
-        "kio_activities", 0, ki18n("Activities"),
-        KDE_VERSION_STRING, KLocalizedString(), KAboutData::License_GPL,
-        ki18n("(c) 2012 Ivan Cukic"));
-
-    setAboutData(about);
-
     d->setupUi(this);
 
     // Plugin selector initialization
@@ -83,24 +74,23 @@ MainConfigurationWidget::MainConfigurationWidget(QWidget *parent, QVariantList a
     d->mainConfig = KSharedConfig::openConfig("kactivitymanagerdrc");
     d->pluginConfig = KSharedConfig::openConfig("kactivitymanagerd-pluginsrc");
 
+    // Loading the plugin selector
     d->pluginSelector = new KPluginSelector(this);
-    d->pluginSelector->addPlugins(
-        plugins,
-        KPluginSelector::ReadConfigFile,
-        i18n("Available Features"),
-        QString(),
-        d->mainConfig);
+    d->pluginSelector->addPlugins(plugins, KPluginSelector::ReadConfigFile,
+                                  i18n("Available Features"), QString(),
+                                  d->mainConfig);
     d->tabWidget->addTab(d->pluginSelector, i18n("Plugins"));
 
     // Keep history initialization
 
     d->spinKeepHistory->setRange(0, INT_MAX);
-
-    d->spinKeepHistory->setSuffix(ki18ncp("unit of time. months to keep the history",
-                                          " month", " months"));
-    d->spinKeepHistory->setPrefix(i18nc("for in 'keep history for 5 months'", "for "));
     d->spinKeepHistory->setSpecialValueText(i18nc("unlimited number of months", "forever"));
 
+    // We don't have KSpingBox anymore, lets keep it alive :)
+    connect(d->spinKeepHistory, SIGNAL(valueChanged(int)),
+            this, SLOT(spinKeepHistoryValueChanged(int)));
+    spinKeepHistoryValueChanged(0);
+
     // Clear recent history button
 
     auto menu = new QMenu(this);
@@ -125,31 +115,32 @@ MainConfigurationWidget::MainConfigurationWidget(QWidget *parent, QVariantList a
     d->checkEnableActivities->setVisible(false);
 
     // Blacklist applications
+
     d->blacklistedApplicationsModel = new BlacklistedApplicationsModel(this);
 
-    QGraphicsScene *scene = new QGraphicsScene(this);
-    d->viewBlacklistedApplications->setScene(scene);
-    QDeclarativeEngine *engine = new QDeclarativeEngine(this);
+    auto layout = new QGridLayout(d->viewBlacklistedApplicationsContainer);
 
-    d->kdeclarative.setDeclarativeEngine(engine);
-    d->kdeclarative.initialize();
-    d->kdeclarative.setupBindings();
-    d->structure = Plasma::PackageStructure::load("Plasma/Generic");
+    d->viewBlacklistedApplications = new QQuickView();
+    d->viewBlacklistedApplications->setColor(
+        QGuiApplication::palette().window().color());
 
-    engine->rootContext()->setContextProperty("applicationModel", d->blacklistedApplicationsModel);
-    QDeclarativeComponent component(engine, QUrl(QString(KAMD_DATA_DIR) + "workspace/settings/BlacklistApplicationView.qml"));
-    // qDebug() << "Errors: " << component.errors();
-    d->viewBlacklistedApplicationsRoot = qobject_cast<QGraphicsObject *>(component.create());
-    d->viewBlacklistedApplicationsRoot->setProperty("width", d->viewBlacklistedApplications->width());
-    scene->addItem(d->viewBlacklistedApplicationsRoot);
+    QWidget *container = QWidget::createWindowContainer(
+        d->viewBlacklistedApplications,
+        d->viewBlacklistedApplicationsContainer);
 
-    d->viewBlacklistedApplications->installEventFilter(this);
+
+    container->setFocusPolicy(Qt::TabFocus);
+    d->viewBlacklistedApplications->rootContext()->setContextProperty(
+        "applicationModel", d->blacklistedApplicationsModel);
+    d->viewBlacklistedApplications->setSource(
+        QStringLiteral(KAMD_INSTALL_PREFIX "/" KAMD_DATA_DIR)
+        + "/workspace/settings/BlacklistApplicationView.qml");
+
+    layout->addWidget(container);
 
     // React to changes
 
-    connect(d->checkEnableActivities, SIGNAL(toggled(bool)), this, SLOT(changed()));
     connect(d->radioRememberAllApplications, SIGNAL(toggled(bool)), this, SLOT(changed()));
-    connect(d->radioRememberSpecificApplications, SIGNAL(toggled(bool)), this, SLOT(changed()));
     connect(d->radioDontRememberApplications, SIGNAL(toggled(bool)), this, SLOT(changed()));
     connect(d->spinKeepHistory, SIGNAL(valueChanged(int)), this, SLOT(changed()));
     connect(d->pluginSelector, SIGNAL(changed(bool)), this, SLOT(changed()));
@@ -157,8 +148,10 @@ MainConfigurationWidget::MainConfigurationWidget(QWidget *parent, QVariantList a
 
     connect(d->radioRememberSpecificApplications, SIGNAL(toggled(bool)),
             d->blacklistedApplicationsModel, SLOT(setEnabled(bool)));
+
     connect(d->radioRememberSpecificApplications, SIGNAL(toggled(bool)),
-            d->viewBlacklistedApplications, SLOT(setEnabled(bool)));
+            d->viewBlacklistedApplicationsContainer, SLOT(setEnabled(bool)));
+
     connect(d->radioRememberSpecificApplications, SIGNAL(toggled(bool)),
             d->checkBlacklistAllNotOnList, SLOT(setEnabled(bool)));
 
@@ -166,26 +159,11 @@ MainConfigurationWidget::MainConfigurationWidget(QWidget *parent, QVariantList a
 
     d->checkBlacklistAllNotOnList->setEnabled(false);
     d->blacklistedApplicationsModel->setEnabled(false);
-    d->viewBlacklistedApplications->setEnabled(false);
+    d->viewBlacklistedApplicationsContainer->setEnabled(false);
 }
 
-void MainConfigurationWidget::updateLayout()
+MainConfigurationWidget::~MainConfigurationWidget()
 {
-    d->viewBlacklistedApplicationsRoot->setProperty("width",
-                                                    d->viewBlacklistedApplications->width() - 32);
-    d->viewBlacklistedApplicationsRoot->setProperty("minimumHeight",
-                                                    d->viewBlacklistedApplications->height() - 32);
-}
-
-bool MainConfigurationWidget::eventFilter(QObject *obj, QEvent *event)
-{
-    if (obj == d->viewBlacklistedApplications) {
-        if (event->type() == QEvent::Resize) {
-            updateLayout();
-        }
-    }
-
-    return false;
 }
 
 void MainConfigurationWidget::defaults()
@@ -202,15 +180,19 @@ void MainConfigurationWidget::load()
     d->pluginSelector->load();
     d->blacklistedApplicationsModel->load();
 
-    const auto statisticsConfig = d->pluginConfig->group("Plugin-org.kde.kactivitymanager.resourcescoring");
+    const auto statisticsConfig = d->pluginConfig->group(
+        "Plugin-org.kde.ActivityManager.ResourceScoring");
+
+    const auto whatToRemember = (WhatToRemember)statisticsConfig.readEntry(
+        "what-to-remember", (int)AllApplications);
 
-    const auto whatToRemember = (WhatToRemember)statisticsConfig.readEntry("what-to-remember", (int)AllApplications);
     d->radioRememberAllApplications->setChecked(whatToRemember == AllApplications);
     d->radioRememberSpecificApplications->setChecked(whatToRemember == SpecificApplications);
     d->radioDontRememberApplications->setChecked(whatToRemember == NoApplications);
 
     d->spinKeepHistory->setValue(statisticsConfig.readEntry("keep-history-for", 0));
-    d->checkBlacklistAllNotOnList->setChecked(statisticsConfig.readEntry("blocked-by-default", false));
+    d->checkBlacklistAllNotOnList->setChecked(
+        statisticsConfig.readEntry("blocked-by-default", false));
 }
 
 void MainConfigurationWidget::save()
@@ -218,15 +200,13 @@ void MainConfigurationWidget::save()
     d->pluginSelector->save();
     d->blacklistedApplicationsModel->save();
 
-    auto statisticsConfig = d->pluginConfig->group("Plugin-org.kde.kactivitymanager.resourcescoring");
+    auto statisticsConfig = d->pluginConfig->group(
+        "Plugin-org.kde.ActivityManager.ResourceScoring");
 
-    WhatToRemember whatToRemember = AllApplications;
-
-    if (d->radioRememberSpecificApplications->isChecked()) {
-        whatToRemember = SpecificApplications;
-    } else if (d->radioDontRememberApplications->isChecked()) {
-        whatToRemember = NoApplications;
-    }
+    const auto whatToRemember =
+        d->radioRememberSpecificApplications->isChecked() ? SpecificApplications :
+        d->radioDontRememberApplications->isChecked()     ? NoApplications :
+        /* otherwise */                                     AllApplications;
 
     statisticsConfig.writeEntry("what-to-remember", (int)whatToRemember);
     statisticsConfig.writeEntry("keep-history-for", d->spinKeepHistory->value());
@@ -234,7 +214,7 @@ void MainConfigurationWidget::save()
 
     auto pluginListConfig = d->mainConfig->group("Plugins");
 
-    pluginListConfig.writeEntry("org.kde.kactivitymanager.resourcescoringEnabled",
+    pluginListConfig.writeEntry("org.kde.ActivityManager.ResourceScoringEnabled",
                                 whatToRemember != NoApplications);
 
     statisticsConfig.sync();
@@ -243,13 +223,10 @@ void MainConfigurationWidget::save()
 
 void MainConfigurationWidget::forget(int count, const QString &what)
 {
-    QDBusInterface rankingsservice(
-        "org.kde.ActivityManager",
-        "/ActivityManager/Resources/Scoring",
-        "org.kde.ActivityManager.Resources.Scoring");
+    KAMD_DECL_DBUS_INTERFACE(rankingsservice, Resources/Scoring, ResourcesScoring);
 
     rankingsservice.asyncCall(
-        "deleteRecentStats", QString(), count, what);
+        "DeleteRecentStats", QString(), count, what);
 }
 
 void MainConfigurationWidget::forgetLastHour()
@@ -272,4 +249,14 @@ void MainConfigurationWidget::forgetAll()
     forget(0, "everything");
 }
 
+void MainConfigurationWidget::spinKeepHistoryValueChanged(int value)
+{
+    static auto months = ki18ncp("unit of time. months to keep the history", " month", " months");
+
+    if (value) {
+        d->spinKeepHistory->setPrefix(i18nc("for in 'keep history for 5 months'", "for "));
+        d->spinKeepHistory->setSuffix(months.subs(value).toString());
+    }
+}
+
 #include "MainConfigurationWidget.moc"
